# This file is part of the egoboost package which is release under the MIT
# License. You can find the full text of the license in the LICENSE file.
# Copyright (C) 2010 Sebastian Rahlf <basti at redtoad dot de>

import cmd
import os, os.path
import optparse
import re
import sys

import storage
import utils

CONFIG_DIR = os.path.expanduser('~/.ego-boost')
DATABASE = 'downloads.db'

class CommandLine(cmd.Cmd):

    """
    Accepts commands via command line.
    """

    def __init__(self, db, quiet=False):
        cmd.Cmd.__init__(self)
        self.db = db
        self.quiet = quiet

    def default(self, line):
        raise NotImplementedError

    def log(self, msg):
        if not self.quiet:
            print >> sys.stderr, msg

    def do_collect(self, line):
        """
        Collects download information for package::

            collect <package> URL [URL ...]
        """
        SOURCE_REG = {
            'bitbucket' : re.compile('(bit|byte)bucket\.org'),
            'pypi' : re.compile('pypi.python.org'),
            'github' : re.compile('github.com'),
        }
        def guess_source(url):
            "Get source identifier from URL."
            for key, reg in SOURCE_REG.items():
                if reg.search(url):
                    return key
            return None

        parts = line.split()
        package = parts.pop(0)
        for url in parts:
            def subst(mobj):
                "Substitute shortcuts (like 'pypi:...')"
                return {
                    'pypi' : 'http://pypi.python.org/pypi/',
                    'bb' : 'http://bitbucket.org/',
                    'gh' : 'http://github.com/',
                }[mobj.group(1)]
            url = re.sub('(^\w+):(?!//)', subst, url)

            source = guess_source(url)

            self.log('Fetching data from %s...' % url)
            data = utils.get_downloads(url)

            self.db.add(package, data, source)

    def do_export(self, line):
        """
        Exports data to CSV.
        """
        print self.db.tocsv()

def main(argv=sys.argv[1:]):
    """
    Main program.
    """
    # create config dir if not exists
    if not os.path.exists(CONFIG_DIR):
        os.mkdir(CONFIG_DIR)

    parser = optparse.OptionParser()
    parser.add_option('-q', '--quiet', dest='quiet', action='store_true',
          help='Suppress output.', default=False)
    opt, arg = parser.parse_args(argv)

    if len(arg) == 0:
        parser.print_help()
        parser.exit()

    db = storage.DB(os.path.join(CONFIG_DIR, DATABASE))
    command = CommandLine(db, opt.quiet)
    
    try:
        command.onecmd(' '.join(arg))
    except NotImplementedError:
        parser.error('Unknown command!')

if __name__ == '__main__':
    main('collect python-weewar pypi:python-weewar bb:basti/python-weewar --quiet'.split())


# This file is part of the egoboost package which is release under the MIT
# License. You can find the full text of the license in the LICENSE file.
# Copyright (C) 2010 Sebastian Rahlf <basti at redtoad dot de>

import lxml.html
import lxml.etree
import re
import xmlrpclib

def get_pypi_downloads(name, show_hidden=True):
    """
    Fetches PyPi download statistics via XMLRPC.

    >>> get_pypi_downloads('python-amazon-product-api')
    {'0.2.0': {'python-amazon-product-api-0.2.0.tar.gz': 68,
               'python-amazon-product-api-0.2.0.win32-py2.5.exe': 15,
               'python-amazon-product-api-0.2.0.win32-py2.5.msi': 17,
               'python-amazon-product-api-0.2.0.win32-py2.6.exe': 17,
               'python-amazon-product-api-0.2.0.win32-py2.6.msi': 19},
     '0.2.1': {'python-amazon-product-api-0.2.1.tar.gz': 130},
     '0.2.2': {'python-amazon-product-api-0.2.2.tar.gz': 101,
               'python-amazon-product-api-0.2.2.win32.exe': 29},
     '0.2.3': {'python-amazon-product-api-0.2.3.tar.gz': 90,
               'python-amazon-product-api-0.2.3.win32-py2.5.exe': 7}}
    """
    client = xmlrpclib.ServerProxy('http://pypi.python.org/pypi')
    def fetch_downloads(rel):
        return dict((data.get('filename'), data.get('downloads'))
            for data in client.release_urls(name, rel))
    return dict((release, fetch_downloads(release))
        for release in client.package_releases(name, show_hidden))

def get_bitbucket_downloads(user, repo):
    """
    Parses bitbucket.org repository download page for download counts.

    >>> get_bitbucket_downloads('basti', 'python-amazon-product-api')
    {'0.2.0': {'python-amazon-product-api-0.2.0.tar.gz': 2303},
     '0.2.1': {'python-amazon-product-api-0.2.1.tar.gz': 2138},
     '0.2.2': {'python-amazon-product-api-0.2.2.tar.gz': 1758},
     '0.2.3': {'python-amazon-product-api-0.2.3.tar.gz': 1138}}
    """
    url = 'http://bitbucket.org/%(user)s/%(repo)s/downloads' % locals()
    root = lxml.html.parse(url).getroot()
    downloads = root.xpath("//tr[starts-with(@id, 'download')]")

    TIMES_REG = re.compile('(\d+) times')
    VERSION_REG = re.compile('(\d+\.\d+(\.\d+)?)')

    counts = {}
    for tag in downloads:
        tds = tag.xpath('td')
        filename = tds[0].text_content().strip()
        try:
            count = int(TIMES_REG.search(tds[4].text_content().strip()).group(1))
            version = VERSION_REG.search(filename).group(1)
            if version not in counts:
                counts[version] = {filename : count}
            else:
                counts[version][filename] = count
        except AttributeError:
            continue
    return counts

def get_github_downloads(user, repo):
    """
    Parses github.com repository download page for download counts.

    >>> get_github_downloads('redtoad', 'python-amazon-product-api')
    {'0.2.0': {'python-amazon-product-api-0.2.0.tar.gz': 2303},
     '0.2.1': {'python-amazon-product-api-0.2.1.tar.gz': 2138},
     '0.2.2': {'python-amazon-product-api-0.2.2.tar.gz': 1758},
     '0.2.3': {'python-amazon-product-api-0.2.3.tar.gz': 1138}}
    """
    url = 'http://github.com/%(user)s/%(repo)s/downloads' % locals()
    root = lxml.html.parse(url).getroot()
    downloads = root.xpath("//tr[starts-with(@id, 'download')]")
    
    TIMES_REG = re.compile('^(\d+)$')
    VERSION_REG = re.compile('(\d+\.\d+(\.\d+)?)')

    counts = {}
    for tag in downloads:
        tds = tag.xpath('td')
        filename = tds[1].text_content().strip()
        try:
            count = int(TIMES_REG.search(tds[4].text_content().strip()).group(1))
            version = VERSION_REG.search(filename).group(1)
            if version not in counts:
                counts[version] = {filename : count}
            else:
                counts[version][filename] = count
        except AttributeError:
            continue
    return counts

PATH_REGS = [
    (re.compile('http://pypi.python.org/pypi/(?P<pkg>[\w-]+)/?'),
        get_pypi_downloads), 
    (re.compile('http://bitbucket.org/(?P<user>\w+)/(?P<repo>[\w-]+)/?'),
        get_bitbucket_downloads),
    (re.compile('http://github.com/(?P<user>\w+)/(?P<repo>[\w-]+)/?'),
        get_github_downloads),
]
    
def get_downloads(url):
    """
    Fetches download statistics from URL. If URL does not match any know 
    pattern, a ``ValueError`` is raised.
    """
    for regex, fnc in PATH_REGS:
        m = regex.search(url)
        if m: return fnc(*m.groups())
    raise ValueError

if __name__ == '__main__':

    import simplejson as json
    import sys

    for path in sys.argv[1:]:
        data = get_downloads(path)
        print json.dumps(data)




# This file is part of the egoboost package which is release under the MIT
# License. You can find the full text of the license in the LICENSE file.
# Copyright (C) 2010 Sebastian Rahlf <basti at redtoad dot de>

"""
Storage for download stats.
"""

import csv
import datetime
import re
import simplejson
import StringIO
import sqlite3
import time


class DB (object):

    """
    Database wrapper object for storing/retrieving data from a SQLite db.
    """

    def __init__(self, path, create_table=True):
        self.conn = sqlite3.connect(path, 
            detect_types=sqlite3.PARSE_DECLTYPES|sqlite3.PARSE_COLNAMES)

        # create table if it does not exist already
        if create_table:
            cur = self.conn.cursor()
            cur.execute('''CREATE TABLE IF NOT EXISTS downloads (
                date DATE,
                package TEXT,
                version TEXT,
                filename TEXT,
                downloads INTEGER, 
                source TEXT,
                CONSTRAINT unique_row UNIQUE (date, package, version, filename, 
                    downloads, source) ON CONFLICT IGNORE
            )''')
            self.conn.commit()

    def __del__(self):
        # close DB connection on garbage collection
        self.conn.close()

    def add(self, package, data, source, date=None):
        """
        Adds download data for package. ``data`` is of format ::

            {
                '0.2.0': {
                    'python-amazon-product-api-0.2.0.tar.gz': 68,
                    'python-amazon-product-api-0.2.0.win32-py2.5.exe': 15,
                    'python-amazon-product-api-0.2.0.win32-py2.5.msi': 17,
                    'python-amazon-product-api-0.2.0.win32-py2.6.exe': 17,
                    'python-amazon-product-api-0.2.0.win32-py2.6.msi': 19},
                '0.2.1': {
                    'python-amazon-product-api-0.2.1.tar.gz': 130},
                '0.2.2': {
                    'python-amazon-product-api-0.2.2.tar.gz': 101,
                    'python-amazon-product-api-0.2.2.win32.exe': 29},
                '0.2.3': {'python-amazon-product-api-0.2.3.tar.gz': 90,
                    'python-amazon-product-api-0.2.3.win32-py2.5.exe': 7}
            }
        
        which is, incidentally, the very same format that comes out of the 
        ``get_*_downloads`` functions.
        """
        if date is None:
            date = datetime.date.today()

        def build_rows():
            "Prepares data for executemany()."
            for version, files in data.items():
                # ignore meta data
                if type(files) != dict:
                    continue
                for filename, count in files.items():
                    yield {
                        'date' : date, 
                        'version' : version, 
                        'package' : package, 
                        'version' : version, 
                        'filename' : filename, 
                        'downloads' : count, 
                        'source' : source
                    }
            
        cur = self.conn.cursor()
        cur.executemany('''INSERT INTO downloads (
            date, package, version, filename, downloads, source) VALUES (
            :date, :package, :version, :filename, :downloads, :source)''',
            build_rows())
        self.conn.commit()

    def date_range(self):
        """
        Returns first and last date for which data has been stored.
        """
        cur = self.conn.cursor()
        cur.execute('''SELECT MIN(date) as "first [date]", 
            MAX(date) as "last [date]" FROM downloads;''')
        return cur.fetchone()

    def count(self):
        """
        Returns total row count.
        """
        cur = self.conn.cursor()
        cur.execute('SELECT COUNT(*) FROM downloads;')
        return cur.fetchone()[0]

    def get_packages(self, ignore_versions=True):
        """
        Gets all packages.
        """
        cur = self.conn.cursor()
        if ignore_versions:
            cur.execute('SELECT DISTINCT package FROM downloads;')
        else:
            cur.execute('SELECT DISTINCT package || " " || version FROM downloads;')
        return [pkg[0] for pkg in cur.fetchall()]

    def tocsv(self, ignore_versions=True):
        """
        Returns download statistics as CSV data. Example::

            date,python-amazon-product-api,python-weewar
            2010-05-04,7834,0
            ...
            2010-05-13,7886,2792
            2010-05-14,7897,2792
        """
        first, last = self.date_range()

        # abort processing for no data
        if not (first or last):
            return ''

        span = (last - first).days
        current = first

        # write CSV headers
        buffer = StringIO.StringIO()
        writer = csv.writer(buffer)
        packages = self.get_packages(ignore_versions)
        writer.writerow(['date'] + packages)

        # collect downloads for per package per date
        cur = self.conn.cursor()
        while current <= last:
            if ignore_versions:
                sql = """SELECT date, package, SUM(downloads)
                FROM downloads 
                WHERE date = ? 
                GROUP BY date, package;
                """
            else:
                sql = """
                SELECT date, package || ' ' || version, SUM(downloads)
                FROM downloads 
                WHERE date = ? 
                GROUP BY date, package, version;
                """

            cur.execute(sql, (current, ))
            data = dict((pkg, count) for date, pkg, count in cur.fetchall())
            writer.writerow([current] + [data.get(pkg, 0) for pkg in packages])
            current += datetime.timedelta(days=+1)

        buffer.flush()
        return buffer.getvalue()


def read_json(fp):
    """
    Loads download data from JSON file. Newer versions store meta data as well::

        {u'0.2.3': {...}, 
        '_package': 'python-amazon-product-api', 
        '_source': 'pypi', 
        '_date': datetime.date(2010, 6, 10)}

    Older versions store this data in the file name itself (eg. 
    ``pypi-20100610-python-amazon-product-api.json``). A ``ValueError`` is 
    raised if this data cannot be extracted.
    """
    data = simplejson.load(fp)
    # Old versions have meta data in file name. If no meta data can be found
    # (ie. _source, _date, _package) try to extract it from the file name.
    if not '_date' in data:
        filename = fp.name
        m = re.search('(?P<_source>\w+)-(?P<_date>\d{8})-'
                      '(?P<_package>[\w-]+)', filename)
        if not m:
            raise ValueError('No meta data found!')
        data.update(m.groupdict())
        data['_date'] = datetime.date(*time.strptime(data['_date'], '%Y%m%d')[:3])
    return data


if __name__ == '__main__':

    # load/create database
    db = DB('/tmp/test.db')
    first, last = db.date_range()
    print '%i rows from %s to %s.' % (db.count(), first, last)

    # show data
    print db.tocsv(False)

#    # fetch data and store in db
#    from utils import get_downloads
#    data = get_downloads('http://bitbucket.org/basti/python-amazon-product-api/')
#    db.add('python-amazon-product-api', data, 'bitbucket')

#    # import old data from JSON files
#    import os, os.path
#    for filename in os.listdir('data'):
#        print 'Loading %s...' % filename
#        try:
#            data = read_json(open(os.path.join('data', filename)))
#            db.add(data['_package'], data, data['_source'], data['_date'])
#        except ValueError, e:
#            print 'ERROR:', e




#!/usr/bin/python
from setuptools import setup

setup(name='ego-boost',
    version='0.1a1',
    description="Package to track you module's download statistics",
    long_description=open('README').read(),
    author='Sebastian Rahlf',
    author_email='basti at redtoad dot de',
    license='MIT', 
    url='http://pypi.python.org/pypi/egoboost',
    packages=['egoboost'],
    scripts=['ego-boost'],
    install_requires=[
        'simplejson',
        'lxml'
    ],
    classifiers = [
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: System :: Archiving :: Packaging',
        'Topic :: System :: Logging', 
    ]
)
